﻿using UnityEngine;
using System.Collections;

// 銃
public class Gun : MonoBehaviour
{

	public enum FireMode
	{
		Auto,
		Burst,
		Single}

	;

	public FireMode fireMode;

	// 銃口・マズル
	public Transform[] projectileSpawn;
	public Projectile projectile;
	public float msBetweenShots = 100;
	public float muzzleVelocity = 35;
	public int burstCount;
	// マガジンに入っている弾の数
	public int projectilesPerMag;
	public float reloadTime = .3f;

	[Header ("Recoil")]
	// x is x-axis
	// y is y-axis
	public Vector2 kickMinMax = new Vector2 (.05f, .2f);
	public Vector2 recoilAngleMinMax = new Vector2 (3, 5);
	// 反動が戻る速度。
	public float recoilMoveSettleTime = .1f;
	// リコイルのX座標の回転が戻る速度
	public float recoilRotationSettleTime = .1f;

	[Header ("Effects")]
	public Transform shell;
	public Transform shellEjection;
	public AudioClip shootAudio;
	public AudioClip reloadAudio;
	MuzzleFlash muzzleflash;

	float nextShotTime;

	bool triggerReleasedSingleLastShot;
	int shotsRemainingInBurst;
	// 現在の残弾数
	int projectilesRemainingInMag;
	bool isReloading;

	Vector3 recoilSmoothDampVelocity;
	float recoilRotSmoothDampVelocity;
	float recoilAngle;

	void Start ()
	{
		muzzleflash = GetComponent<MuzzleFlash> ();
		shotsRemainingInBurst = burstCount;
	}
	// Aim関数はUpdate時にRotateを変更するため、
	void LateUpdate ()
	{
		// animate recoil
		transform.localPosition = Vector3.SmoothDamp (transform.localPosition, Vector3.zero, ref recoilSmoothDampVelocity, recoilMoveSettleTime);
		recoilAngle = Mathf.SmoothDamp (recoilAngle, 0, ref recoilRotSmoothDampVelocity, recoilRotationSettleTime);
		// TODO このアルゴリズム理解すること
		// leftはX軸に対して回転
		transform.localEulerAngles = transform.localEulerAngles + Vector3.left * recoilAngle; // Vector3.right * -recoilAngleでも同様。


		// 自動リロード
		if (!isReloading && projectilesRemainingInMag == 0) {
			Reload ();
		}
	}

	void Shoot ()
	{
		// リロード中でなく、弾を打てる状態で、弾があるとき、
		if (!isReloading && Time.time > nextShotTime && projectilesRemainingInMag > 0) {

			if (fireMode == FireMode.Burst) {
				if (shotsRemainingInBurst == 0) {
					return;
				}
				shotsRemainingInBurst--;
			} else if (fireMode == FireMode.Single) {
				if (!triggerReleasedSingleLastShot) {
					return;
				}
			}

			for (int i = 0; i < projectileSpawn.Length; i++) {
				if (projectilesRemainingInMag == 0) {
					break;
				}
				projectilesRemainingInMag--;
				// Shot
				nextShotTime = Time.time + msBetweenShots / 1000;
				Projectile newProjectile = Instantiate (projectile, projectileSpawn [i].position, projectileSpawn [i].rotation) as Projectile;
				newProjectile.SetSpeed (muzzleVelocity);
			}
			// Shell
			Instantiate (shell, shellEjection.position, shellEjection.rotation);
			// muzzle flash
			muzzleflash.Activate ();

			// Recoil リコイル
			transform.localPosition -= Vector3.forward * Random.Range (kickMinMax.x, kickMinMax.y);
			recoilAngle += Random.Range (recoilAngleMinMax.x, recoilAngleMinMax.y);
			recoilAngle = Mathf.Clamp (recoilAngle, 0, 30); // max 30

			AudioManager.instance.PlaySound (shootAudio, transform.position);
		}
	}

	public void Reload ()
	{
		if (!isReloading && projectilesRemainingInMag != projectilesPerMag) {
			StartCoroutine (AnimateReload ());
			AudioManager.instance.PlaySound (reloadAudio, transform.position);
		}
	}

	IEnumerator AnimateReload ()
	{
		isReloading = true;

		yield return new WaitForSeconds (.2f);

		float reloadSpeed = 1 / reloadTime;
		float percent = 0;
		Vector3 initialRot = transform.localEulerAngles;
		float maxReloadAngle = 30;

		while (percent < 1) {
			percent += Time.deltaTime * reloadSpeed;

			// 二次関数で-1なので、XY座標で山を描く(放物線のような)補間。
			float interpolation = (-Mathf.Pow (percent, 2) + percent) * 4;
			float reloadAngle = Mathf.Lerp (0, maxReloadAngle, interpolation);
			transform.localEulerAngles = initialRot + Vector3.left * reloadAngle;

			yield return null;
		}

		isReloading = false;
		projectilesRemainingInMag = projectilesPerMag; // 弾補充
	}

	public void Aim (Vector3 aimPoint)
	{
		if (!isReloading) {
			transform.LookAt (aimPoint);
		}
	}

	public void OnTriggerHold ()
	{
		// 現在はPlayer.csがUpdate時に呼び出す。
		// よって、以下ShootはUpdate時に呼ばれている。この時にリコイルの計算をして、
		// LastUpdate時に実際に動かす。
		Shoot ();
		triggerReleasedSingleLastShot = false;
	}

	public void OnTriggerRelease ()
	{
		triggerReleasedSingleLastShot = true;
		shotsRemainingInBurst = burstCount;
	}

}
