﻿using UnityEngine;
using System.Collections;

[RequireComponent (typeof(PlayerController))]
[RequireComponent (typeof(GunController))]
public class Player : LivingEntity
{
	public float moveSpeed = 5;

	public Crosshairs crosshairs;

	Camera viewCamera;
	PlayerController controller;
	GunController gunController;

	void Awake ()
	{
		controller = GetComponent<PlayerController> ();
		gunController = GetComponent<GunController> ();
		viewCamera = Camera.main;
		FindObjectOfType<Spawner> ().OnNewWave += OnNewWave;
	}

	protected override void Start ()
	{
		base.Start ();
	}

	void OnNewWave (int waveNumber)
	{
		health = startingHealth;
		gunController.EquipGun (waveNumber - 1);
	}

	void Update ()
	{
		// 移動用入力
		Vector3 moveInput = new Vector3 (Input.GetAxisRaw ("Horizontal"), 0, Input.GetAxisRaw ("Vertical"));
		// TODO why normailze?
		Vector3 moveVelocity = moveInput.normalized * moveSpeed;
		controller.Move (moveVelocity);
	
		// プレイヤーの向きを変更するロジック
		// カメラからマウスポイントに向かう光線
		Ray ray = viewCamera.ScreenPointToRay (Input.mousePosition);
		Plane groundPlane = new Plane (Vector3.up, Vector3.up * gunController.GunHeight);
		//Plane groundPlane = new Plane (Vector3.up, Vector3.zero);
		float rayDistance;
		if (groundPlane.Raycast (ray, out rayDistance)) { // 平面と光線が交差したところまでの光線の距離が取得できる。
			Vector3 point = ray.GetPoint (rayDistance); // Ray光線は、直線がずっと続いているので、分かった距離の座標を取得
			Debug.DrawLine (ray.origin, point, Color.red);
			controller.LookAt (point);

			// 照準をマウスポインターに合わせる
			crosshairs.transform.position = point;
			crosshairs.DetectTargets (ray);

			// 1より距離が大きい場合は、補正を行う。マウスカーソルが近い場合は、銃の向き補正をすると
			// 銃を横に向けるため、1より小さい場合は、補正を行わない。
			if ((new Vector2 (point.x, point.z) - new Vector2 (transform.position.x, transform.position.z)).sqrMagnitude > 1 * 1) {
				// 補正。銃を右手に持っているので、照準があっていない。
				gunController.Aim (point);
			}
			//Debug.Log ((new Vector2 (point.x, point.z) - new Vector2 (transform.position.x, transform.position.z)).magnitude);
		}

		// 武器用入力
		// TODO: Input.GetMouseButton(0) と変えた。
		if (Input.GetButton ("Fire1")) {
			gunController.OnTriggerHold ();
		}
		if (Input.GetButtonUp ("Fire1")) {
			gunController.OnTriggerRelease ();
		}

		if (Input.GetKeyDown (KeyCode.R)) {
			gunController.Reload ();
		}

		if (transform.position.y < -10) {
			TakeDamage (health);
		}
	}

	public override void Die ()
	{
		AudioManager.instance.PlaySound ("Player Death", transform.position);
		base.Die ();
	}
}
