﻿using UnityEngine;
using System.Collections;

// 敵のウェーブを作る
public class Spawner : MonoBehaviour
{
	public bool developMode;

	public Wave[] waves;
	public Enemy enemy;

	LivingEntity playerEntity;
	Transform playerT;

	Wave currentWave;
	int currentWaveNumber;

	// 1ウェーブ中のまだ生成してない敵の数
	int enemiesRemainingToSpawn;
	// 1ウェーブ内で、すでに倒した数
	int enemiesRemainingAlive;
	// 次の敵が出現するスポーン時間
	float nextSpawnTime;

	MapGenerator map;

	float timeBetweenCampingChecks = 2;
	// 2 seconds.
	float campThreshouldDistance = 1.5f;
	float nextCampCheckTime;
	Vector3 campPositionOld;
	bool isCamping;

	bool isDisabled;

	public event System.Action<int> OnNewWave;

	void Start ()
	{
		playerEntity = FindObjectOfType<Player> ();
		playerT = playerEntity.transform;

		nextCampCheckTime = timeBetweenCampingChecks + Time.time;
		campPositionOld = playerT.position;
		playerEntity.OnDeath += OnPlayerDeath;

		// シーン全体からMapGenerator型のオブジェクトを探す。
		map = FindObjectOfType<MapGenerator> ();
		NextWave ();
	}

	void Update ()
	{
		if (!isDisabled) {
			// キャンプチェック
			if (Time.time > nextCampCheckTime) {
				nextCampCheckTime = timeBetweenCampingChecks + Time.time;
				isCamping = (Vector3.Distance (playerT.position, campPositionOld) < campThreshouldDistance);
				campPositionOld = playerT.position;
			}

			if ((enemiesRemainingToSpawn > 0 || currentWave.infinite) && Time.time > nextSpawnTime) {
				enemiesRemainingToSpawn--;
				nextSpawnTime = Time.time + currentWave.timeBetweenSpawns;

				StartCoroutine ("SpawnEnemy");
			}			
		}

		if (developMode) {
			if (Input.GetKeyDown (KeyCode.Return)) {
				StopCoroutine ("SpawnEnemy");
				foreach (Enemy enemy in FindObjectsOfType<Enemy>()) {
					GameObject.Destroy (enemy.gameObject);
				}
				NextWave ();
			}
		}
	}

	IEnumerator SpawnEnemy ()
	{
		float spawnDelay = 1;
		float tileFlashSpeed = 4;

		Transform spawnTile = map.GetRandomOpenTile ();
		if (isCamping) {
			spawnTile = map.GetTileFromPosition (playerT.position);
		}
		Material tileMat = spawnTile.GetComponent<Renderer> ().material;
		Color initialColour = Color.white;
		Color flashColour = Color.red;
		float spawnTimer = 0;

		// Flash color animation
		while (spawnTimer < spawnDelay) {

			tileMat.color = Color.Lerp (initialColour, flashColour, Mathf.PingPong (spawnTimer * tileFlashSpeed, 1));

			spawnTimer += Time.deltaTime;
			yield return null;
		}

		Enemy spawnedEnemy = Instantiate (enemy, spawnTile.position + Vector3.up, Quaternion.identity) as Enemy;
		spawnedEnemy.OnDeath += OnEnemyDeath; // 敵を生成したときに、eventハンドラも登録。
		// TODO: これでは複数の敵の場合に、ヘルスが固定になってしまうのでは?
		spawnedEnemy.SetCharacteristics (currentWave.moveSpeed, currentWave.hitsToKillPlayer, currentWave.enemyHealth, currentWave.skinColour);
	}

	void OnPlayerDeath ()
	{
		isDisabled = true;
	}

	void OnEnemyDeath ()
	{
		enemiesRemainingAlive--;

		if (enemiesRemainingAlive == 0) {
			NextWave ();
		}
	}

	void ResetPlayerPosition ()
	{
		playerT.position = map.GetTileFromPosition (Vector3.zero).position + Vector3.up * 3;
	}

	void NextWave ()
	{
		if (currentWaveNumber > 0) {
			AudioManager.instance.PlaySound2D ("Level Complete");
		}

		currentWaveNumber++;
		if (currentWaveNumber - 1 < waves.Length) {
			currentWave = waves [currentWaveNumber - 1];

			enemiesRemainingToSpawn = currentWave.enemyCount;
			enemiesRemainingAlive = enemiesRemainingToSpawn;

			if (OnNewWave != null) {
				OnNewWave (currentWaveNumber);
			}
			ResetPlayerPosition ();
		}
	}

	[System.Serializable]
	public class Wave
	{
		public bool infinite;
		public int enemyCount;
		// 出現間隔
		public float timeBetweenSpawns;

		public float moveSpeed;
		public int hitsToKillPlayer;
		public float enemyHealth;
		public Color skinColour;
	}
}
